const stringify = require('json-stringify');
const utils = require('./utils');
const logger = utils.getLogger();

const LIST_OF_ACTIONS = {
  PAUSE_CONSUMER: 1,
  PLAY_CONSUMER: 2,
};

const busController = function(bus) {
  return async function({message}) {
    let value = message.value.toString();
    logger.debug('Received message on control stream', value);
    try {
      let action = JSON.parse(value);
      if (!action || !action.type) await Promise.reject('Null action');

      switch (action.type) {
        case LIST_OF_ACTIONS.PAUSE_CONSUMER:
          await pauseConsumer(bus, action);
          break;
        case LIST_OF_ACTIONS.PLAY_CONSUMER:
          await playConsumer(bus, action);
          break;
        default:
          await Promise.reject('Unknown action supplied.\n' + action);
      }
    } catch (e) {
      logger.error({e}, 'Error in executing control action.', stringify(value));
    }
  };
};

// default pause obj would be
// {type: 1, target: 'service-name', source: 'source-service-name', event: 'event-name'}
async function pauseConsumer(bus, action) {
  if (action.target === bus._serviceName) {// check if current is the target
    const topic = utils.getTopicName(action.source, action.event);
    logger.info('Received pause event for topic', topic);
    if (!bus.listOfWrapConsumers[topic])
      await Promise.reject('Unsubscribed topic supplied');

    bus.listOfWrapConsumers[topic].pauseConsumption();
  }
}

// default play obj would be
// {type: 2, target: 'service-name', source: 'source-service-name', event: 'event-name'}
async function playConsumer(bus, action) {
  if (action.target === bus._serviceName) {// check if current is the target
    const topic = utils.getTopicName(action.source, action.event);
    logger.info('Received play event for topic', topic);
    if (!bus.listOfWrapConsumers[topic])
      await Promise.reject('Unsubscribed topic supplied');

    bus.listOfWrapConsumers[topic].resumeConsumption();
  }
}

module.exports = busController;