/*
  ** NOTE**
  * EventBus class will be exposed out to client, so dont expose or leave
  * any direct access to kafka or any internal objects
 */

// NOTE: no imports above here, any imports do inside EventBus class
function EventBus(serviceName, config) {
  if (!serviceName || typeof serviceName !== 'string')
    throw new Error('Service name is required');
  serviceName = serviceName.replace('-', '_');
  // setting process env variable before importing anything
  process.env._EVENT_BUS_SERVICE_NAME = serviceName;
  process.env._EVENT_BUS_LOG_LEVEL = config.logLevel || 'info';
  const logger = require('./utils').getLogger();
  // create bus
  const InterfaceBus = require('./busInterface');
  const bus = new InterfaceBus(serviceName, config);
  // start bus controller, this will also test connection to kafka
  bus.startBusController().catch(err => {
    // error means, client failed to connect to server
    // so lets disable everything
    logger.error('Error starting bus controller', err);
    bus.close();
  });
  this.initProducer = (...args) => bus.initProducer(...args);
  this.registerConsumer = (...args) => bus.registerConsumer(...args);
  this.close = (...args) => bus.close(...args);
}

module.exports = EventBus;