const Cassandra = require('cassandra-driver');
const cassandraConnectionMap = {};

module.exports.initConnectionPool = function(config, key) {
  if (!key) key = config.keyspace;
  if (!key) throw new Error('Key or keyspace is empty');
  if (cassandraConnectionMap[key] == null)
    cassandraConnectionMap[key] = new Cassandra.Client(config);
};

module.exports.getDbConnection = function(key) {
  let cP = cassandraConnectionMap[key];
  if (!cP) throw new Error('Connection pool is not initialized');
  return {
    query: (query, values, options = {}, cb) => {
      if (typeof values === 'function') {
        cb = values;
        values = [];
      }
      if (typeof options === 'function') {
        cb = options;
        options = {};
      }
      options.prepare = true;
      return cP.execute(query, values, options, cb);
    },
    batchCUD: (queries, options, cb) => {
      if (typeof options === 'function') {
        cb = options;
        options = {prepare: true};
      } else if (typeof cb === 'function') options.prepare = true;
      return cP.batch(queries, options, cb);
    },
    stream: (query, values, options = {}, cb) => {
      if (typeof values === 'function') {
        cb = values;
        values = [];
      }
      if (typeof options === 'function') {
        cb = options;
        options = {};
      }
      options.prepare = true;
      return cP.stream(query, values, options, cb);
    }
  };
};