const {MongoClient} = require('mongodb');
const dbConnMap = {};
const wrapperMap = {};
module.exports.initConnectionPool = function(dbConfig, cb) {
    let key = dbConfig.KEY;
    let dbName = dbConfig.DB
    let url = "mongodb://"+dbConfig.USER+":"+dbConfig.PASS+"@"+"mongo-casual-hh-1.prod" +
        ":27017/"+dbName
    return new Promise((resolve, reject) => {
        if (dbName == null) {
            if (typeof cb === 'function')
                return cb('Error bootstrapping mongo, dbName missing', null);
            return reject('Error bootstrapping mongo, dbName missing');
        }

        if (typeof key === 'function' || key == null) {
            cb = key;
            key = dbName;
        }
        if (dbConnMap[key] == null) {
            let client = new MongoClient(url,dbConfig.CONNECTION_CONFIG);
            let promise = dbConnMap[key] = client.connect();
            promise.then(() => {
                dbConnMap[key] = client.db(dbName);
                if (typeof cb === 'function') return cb(null, null);
                resolve();
            }).catch(err => {
                if (typeof cb === 'function') return cb(err, null);
                reject(err);
            });
        }
    });
};

/**
 * Reroutes default functions of cluster into dummy wrapper object i.e,
 * dummy object becomes wrapper for db connection object.
 * @param dbConn mongodb connection object
 */
const dummify = function(dbConn) {
    return {
        // Set dbconn as the prototype of wrapper
        __proto__: dbConn,

        // Override hasOwnProperty to prevent calling dbConn's implementation
        hasOwnProperty: function(key) {
            return Object.prototype.hasOwnProperty.call(this, key);
        },
    };
};

module.exports.getDbConnection = function(key, cb) {
    if (wrapperMap[key]) return cb(null, wrapperMap[key]);
    console.log("key mongo check", key)
    let dbConn = dbConnMap[key];
    if (!dbConn)
        throw new Error('Connection pool is not initialized.');
    Promise.resolve(dbConn).
    then(_ => {
        wrapperMap[key] = dummify(dbConnMap[key]);
        cb(null, wrapperMap[key]);
    }).
    catch(err => cb(err));
};