const mongo = require('./mongo');
const loggerPack = require('../logger/logger');

let mongoKey;

function MongoGameLoader(config) {
  let logger = loggerPack(config);
  let mongoConfig = config.MONGO;
  mongoKey = mongoConfig.KEY;
  mongo.initConnectionPool(mongoConfig.URL, mongoConfig.DB, mongoConfig.KEY)
  /**
   * Fetches the game object depending on conditions
   * (gameId, gamePrefix, gameName) from 'games' mongo collection
   * @param gameDetails can be like {gameId: 206} or {gamePrefix: 'rbr'},
   * @return gameObject
   */
  this.getGameObject = function(gameDetails) {
    return new Promise((resolve, reject) => {
      mongo.getDbConnection(mongoKey, async (err, mongoClient) => {
        if (err) {
          reject({err: 'Error connecting to mongo' + err.toString()});
          return;
        }
        if (!gameDetails ||
            Object.keys(gameDetails).length === 0 ||
            (!gameDetails.gameId && !gameDetails.gameName &&
                !gameDetails.gamePrefix)) {
          reject({err: 'Valid argument required for fetching in collection'});
          return;
        }

        let res = await mongoClient.collection('games').
            findOne(gameDetails);
        resolve(res);
      });
    });
  };

  /**
   * Fetches the game objects from 'games' mongo collection
   * @return gameObjects array
   */
  this.getAllGameObjects = function() {
    return new Promise((resolve, reject) => {
      mongo.getDbConnection(mongoKey, async (err, mongoClient) => {
        if (err) {
          reject({err: 'Error connecting to mongo' + err.toString()});
          return;
        }
        let res = await mongoClient.collection('games').
            find({}).
            toArray();
        resolve(res);
      });
    });
  };
};

module.exports = MongoGameLoader;
